#include <avr/io.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "PCD8544.h"

/* LCD cache buffer where the data is store until print */
static unsigned char LcdCache[LCD_CACHE_SIZE];

void LcdInit(void)
{
    /* Pull-up on reset pin. */
    LCD_PORT |= _BV(LCD_RST_PIN);
    LCD_PORT |= _BV(LIGHT_PIN);

    /* Set output bits on LCD Port. */
    LCD_DDR |= _BV(LIGHT_PIN) | _BV(LCD_RST_PIN) | _BV(LCD_DC_PIN)| _BV(LCD_CE_PIN) 
            | _BV(SPI_MOSI_PIN) | _BV(SPI_CLK_PIN);

    Delay();

    /* Toggle display reset pin. */
    LCD_PORT &= ~(_BV(LCD_RST_PIN));
    Delay();
    LCD_PORT |= _BV(LCD_RST_PIN);
    


    /* Enable SPI port:
     * No interrupt, MSBit first, Master mode, CPOL->0, CPHA->0, Clk/4
     */
    SPCR = 0x50;

    /* Disable LCD controller */
    LCD_PORT |= _BV(LCD_CE_PIN);

    LcdSend(0x21, LCD_CMD); /* LCD Extended Commands. */
    LcdSend(0xC8, LCD_CMD); /* Set LCD Vop (Contrast).*/
    LcdSend(0x06, LCD_CMD); /* Set Temp coefficent. */
    LcdSend(0x13, LCD_CMD); /* LCD bias mode 1:48. */
    LcdSend(0x20, LCD_CMD); /* LCD Standard Commands,Horizontal addressing mode */
    LcdSend(0x0C, LCD_CMD); /* LCD in normal mode. */

    /* Clear display on first time use */
    LcdClear();

    /* Turn lights on */
    LCD_PORT &= ~(_BV(LIGHT_PIN));
}

void LcdContrast(unsigned char contrast)
{
    /* LCD Extended Commands. */
    LcdSend(0x21, LCD_CMD);

    /* Set LCD contrast level. */
    LcdSend(0x80 | contrast, LCD_CMD);

    /* LCD Standard Commands, horizontal addressing mode. */
    LcdSend(0x20, LCD_CMD);
}

void LcdClear(void)
{
	/* Clear lcd cache buffer */
	memset(LcdCache, 0x00, LCD_CACHE_SIZE);
}

void LcdPixel(unsigned char x, unsigned char y, LcdPixelMode mode)
{
    int  index;
    unsigned char  offset;
    unsigned char  data;

    /* Prevent from getting out of border */
    if (x > LCD_X_RES) return;
    if (y > LCD_Y_RES) return;

    /* Recalculating index and offset */
    index = ((y / 8) * 84) + x;
    offset = y - (( y / 8) * 8);

    data = LcdCache[index];

    /* Bit processing */

	/* Clear mode */
    if (mode == PIXEL_OFF) {
        data &= (~(0x01 << offset));
    }
    /* On mode */
    else if (mode == PIXEL_ON) {
        data |= (0x01 << offset);
    }
    /* Xor mode */
    else if (mode == PIXEL_XOR) {
        data ^= (0x01 << offset);
    }

    /* Final result copied to cache */
    LcdCache[index] = data;
}

/* Send the cached data to the LCD */
void LcdUpdate(void)
{
    int i;

    LcdSend(0x80, LCD_CMD);
    LcdSend(0x40, LCD_CMD);

    /* Serialize the display buffer. */
    for (i = 0; i < LCD_CACHE_SIZE; ++i)
        LcdSend(LcdCache[i], LCD_DATA);
}

/* Setup the SPI register and send the data */
void LcdSend(unsigned char data, LcdCmdData cd)
{
    /*  Enable display controller (active low). */
    LCD_PORT &= ~(_BV(LCD_CE_PIN));

    if (cd == LCD_DATA)
        LCD_PORT |= _BV(LCD_DC_PIN);
    else
        LCD_PORT &= ~(_BV(LCD_DC_PIN));

    /*  Send data to display controller. */
    SPDR = data;

    /*  Wait until Tx register empty. */
    while ((SPSR & 0x80) != 0x80);

    /* Disable display controller. */
    LCD_PORT |= _BV(LCD_CE_PIN);
}

/* Print the char from the lookup table at position x and y */
void LcdChr(unsigned char ch, unsigned char x, unsigned char y)
{
    unsigned char i, j, col;
	
	if ((y > LCD_Y_RES) || (x > LCD_X_RES))
		return;

    if ((ch < 0x20) || (ch > 0x7b)) {
        /* Convert to a printable character. */
        ch = 92;
    }

    for (i = 0; i < 5; ++i) {
		/* Copy lookup table from Flash ROM to LcdCache */
		col = FontLookup[ch - 32][i];

		for (j = 0; j < 8; ++j) {
			if ((1 << j) & col)
				LcdPixel(x + i, y + j, PIXEL_ON);
			else
				LcdPixel(x + i, y + j, PIXEL_OFF);
		
		}
    }
}

/* Print the string beginig from position x and y */
void LcdStr(unsigned char *str, unsigned char x, unsigned char y)
{
    unsigned char i = 0;
	
	if ((y > LCD_Y_RES) || (x > LCD_X_RES))
		return;

	while (*str != '\0') {
		LcdChr(*str, x + i * 5, y);
		++i;
		++str;
	}
}

/* Uncalibrated delay for LCD init routine. */
void Delay(void)
{
    int i;

    for (i = -32000; i < 32000; ++i);
}

